<?php
/**
 * Copyright 2025 Flex Clam Server for Plesk
 *
 * Permission is hereby not granted to manipulate, modify or adapt the code;
 * as well as obtain a copy of this software or the associated documentation.
 * It is not allowed to publish, distribute, sublicense and/or sell copies of the Software,
 * without the consent of the author or creator of the software.
 *
 * The above copyright notice and this permission notice will be included
 * on all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT
 * AUTHORS OR COPYRIGHT HOLDERS WILL BE RESPONSIBLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION,
 * TORT OR OTHERWISE, ARISING OUT OF OR IN CONNECTION WITH THE SOFTWARE OR USE.
 *
 * @copyright 2025 Flex Clam Server for Plesk
 * @author Flex Antivirus <info@antivirus.flexservices.app>
 * @license Copyright
 */

class Modules_FlexClamServer_Check{

    /**
     * Start Check Step to init
     *
     * @param $domainId		Numeric ID
     */
	public static function Start($domainId , $idTask){

		pm_Log::info("FlexClamServer : Scan " . pm_Domain::getByDomainId($domainId)->getDisplayName());

		$log = Modules_FlexClamServer_Helper::TempPathLog() . pm_Domain::getByDomainId($domainId)->getDisplayName() . ".log";
		$path = str_replace("/", "\\", pm_Domain::getByDomainId($domainId)->getHomePath());
		$Status = pm_ApiCli::callSbin("FlexClamServer.exe",array("-task",$path,$log,"-json"),1);
		if($Status['code']>0){throw new pm_Exception($Status['stderr']);}
		pm_Domain::getByDomainId($domainId)->setSetting("idTask",$idTask);

	}

    /**
     * Get Status progress
     *
     * @param $domainId		Numeric ID
     *
     * @return Array(2)		Percent of progress , Total Files to Checks
     */
	public static function Status($domainId){

		$logPath = Modules_FlexClamServer_Helper::TempPathLog() . pm_Domain::getByDomainId($domainId)->getDisplayName() . ".log";

		$fileManager = new pm_ServerFileManager();
		if (!$fileManager->fileExists($logPath)) {
			return [100,0];
		}else{
			do {
				if (!$fileManager->fileExists($logPath)) {
					break;
				}
				$log = json_decode($fileManager->fileGetContents($logPath));
				
			} while ($log == NULL);

			if($log == NULL){return [100,0];}

			if ($log->status==="task"){
				return [-1,0]; 
			} elseif ($log->status==="end"){
				 return [100,0]; 
			} else {
				 return [((double)$log->checks * 100) / (double)$log->files,$log->files]; 
			}
		}

	}

    /**
     * Get Information Task
     *
     * @param $domainId		Numeric ID
     *
     * @return Array		Status Item
     * @return Array		Data Informacion if task was end
     */
	public static function Info($domainId){

		$logPath = Modules_FlexClamServer_Helper::TempPathLog() . pm_Domain::getByDomainId($domainId)->getDisplayName() . ".log";

		$fileManager = new pm_ServerFileManager();
		if (!$fileManager->fileExists($logPath)) {
			return ["status" => "none", "task" => pm_Domain::getByDomainId($domainId)->getSetting("idTask",0)];
		}else{

			$log = json_decode($fileManager->fileGetContents($logPath));

			if($log->status=="end"){
				return ["status" => $log->status, "task" => pm_Domain::getByDomainId($domainId)->getSetting("idTask",0), "checks" => $log->checks, "virus" => $log->virus, "warning" => [$log->warning]];
			}else{
				return ["status" => $log->status , "task" => pm_Domain::getByDomainId($domainId)->getSetting("idTask",0)];
			}
		}

	}

    /**
     * Stop Checking and delete log
     *
     * @param $domainId		Numeric ID
     *
     * @return Boolean		True
     */
	public static function Stop($domainId){

		pm_Settings::set('scan_lock',0);
		pm_Domain::getByDomainId($domainId)->setSetting("idTask",0);
		Modules_FlexClamServer_Helper::ScanStatusFlag($domainId, false);

		$logPath = Modules_FlexClamServer_Helper::TempPathLog() . pm_Domain::getByDomainId($domainId)->getDisplayName() . ".log";

		$fileManager = new pm_ServerFileManager();
		if (!$fileManager->fileExists($logPath)) {
			return true;
		}else{

			$log = json_decode($fileManager->fileGetContents($logPath));

			if ($log->status==="end"){
				return true;
			} else {
				$fileManager->removeFile($logPath);
				return true;
			}
		}

	}

    /**
     * Get Task from ID
     *
     * @param $idTask		Numeric ID of Task
     *
     * @return Task Class(pm_LongTask_Task) or NULL if not exists
     */
	public static function Task($idTask){

		if($idTask==0){
			return null;
		}else{
		        $taskManager = new pm_LongTask_Manager();

			foreach ($taskManager->getTasks(["task_scan"]) as $task) {
				if ($task->getInstanceId() == $idTask){
					return  $task;
				}

			}
			return null;			
		}	

	}

    /**
     * Get if Task is running
     *
     * @param $idTask		Numeric ID of Task
     *
     * @return Boolean 
     */
	public static function TaskRunning($idTask){
	
		$status = true;

		if((int)$idTask == 0){
			$status=false;
		}else{

		        $taskManager = new pm_LongTask_Manager();
			$foundtask = false;
			foreach ($taskManager->getTasks(["task_scan"]) as $task) {
				if ($task->getInstanceId() == $idTask){
					$foundtask = true;
					if ($task->getStatus() == pm_LongTask_Task::STATUS_DONE || $task->getStatus() == pm_LongTask_Task::STATUS_ERROR  ){ $status=false; }
					break;
				}
			}

			if(!$foundtask){
				foreach ($taskManager->getTasks(["task_mail"]) as $task) {
					if ($task->getInstanceId() == $idTask){
						$foundtask = true;
						if ($task->getStatus() == pm_LongTask_Task::STATUS_DONE || $task->getStatus() == pm_LongTask_Task::STATUS_ERROR  ){ $status=false; }
						break;
					}
				}
			}

			if(!$foundtask){$status=false;}
	
		}	

		return $status;

	}

    /**
     * Get All Task
     *
     * @return Array	numeric idTask from Domains
     */
	public static function AllTask($idTask){
		$dataId = [];
		$Domainlist = Modules_FlexClamServer_Helper::GetAllDomains();
		foreach ($Domainlist as $domain) {
			if($domain['idTask'] === 0){continue;}
			$dataId = [$domain['idTask']];
		}

		return $dataId;
	}

    /**
     * Get Task from Domain
     *
     * @return Numeric	idTask
     */
	public static function DomainTask($domainId){
		return pm_Domain::getByDomainId($domainId)->getSetting("idTask",0);
	}

}